TrailerSound = {};
TrailerSound.dir = g_currentModDirectory;
TrailerSound.volScale = 3
TrailerSound.ptcScale = 1

print("=== TrailerSound v0.2 by mogli & charly_99 ===")

function TrailerSound.prerequisitesPresent(specializations)
	return true
end;

function TrailerSound:load(xmlFile)
	self.trailerSound = {}
	self.trailerSound.count = 0
	self.trailerSound.sumDt = 0
	self.trailerSound.brake = false
	
	if self.isClient then
		local filename = Utils.getFilename( "trailerSound.wav" , TrailerSound.dir )
		self.trailerSound.trailerSample = createSample("trailerSound")
		loadSample(self.trailerSound.trailerSample, filename, false)
		setSampleVolume( self.trailerSound.trailerSample, 0 )
		setSamplePitch(  self.trailerSound.trailerSample, 0.5 )
		
		filename = Utils.getFilename( "surren.wav" , TrailerSound.dir )
		self.trailerSound.wheelSample = createSample("wheelSound")
		loadSample(self.trailerSound.wheelSample, filename, false)
		setSampleVolume( self.trailerSound.wheelSample, 0 )
		setSamplePitch(  self.trailerSound.wheelSample, 0.5 )
		
		filename = Utils.getFilename( "auflaufbremse.wav" , TrailerSound.dir )
		self.trailerSound.brakeSample = createSample("brakeSound")
		loadSample(self.trailerSound.brakeSample, filename, false)
		setSampleVolume( self.trailerSound.brakeSample, 1 )
		setSamplePitch(  self.trailerSound.brakeSample, 1 )
	end
end;

function TrailerSound:delete()
	if self.trailerSound ~= nil then
		if self.trailerSound.trailerSample ~= nil then
			delete( self.trailerSound.trailerSample )			
		end
		if self.trailerSound.wheelSample ~= nil then
			delete( self.trailerSound.wheelSample )			
		end
		if self.trailerSound.brakeSample ~= nil then
			delete( self.trailerSound.brakeSample )			
		end
	end
end;

function TrailerSound:mouseEvent(posX, posY, isDown, isUp, button)
end;

function TrailerSound:keyEvent(unicode, sym, modifier, isDown)
end;

function TrailerSound:readUpdateStream(streamId, timestamp, connection)
end;
 
function TrailerSound:writeUpdateStream(streamId, connection, dirtyMask)
end;
 
function TrailerSound:update(dt)

end;

function TrailerSound:updateTick(dt)	
	--if self.isClient and self.trailerSound.sumDt > 100 and self:getIsActiveForSound() then
	--	if self.trailerSound.sumDt > 200 then
	--		self.trailerSound.sumDt = self.trailerSound.sumDt % 100
	--	else
	--		self.trailerSound.sumDt = self.trailerSound.sumDt - 100
	--	end
		
	if self:getIsActiveForSound() then

		self.trailerSound.sumDt = self.trailerSound.sumDt + dt
		if self.trailerSound.sumDt > 1000 then
			self.trailerSound.count = TrailerSound.countTrailers( self )
			self.trailerSound.sumDt = self.trailerSound.sumDt - 1000
		end			

		local kmh     = math.min( math.max( self.lastSpeed * self.speedDisplayScale * 3600, 0), 100 )	
		local x,_,z   = getWorldTranslation( self.components[1].node )
		local isfield = TrailerSound.isField( x, z )
		if not isfield and kmh > 20 then
			local vol = math.min( math.max( kmh - 20, 0 ) / 30, 1.5 )
			local ptc = 1 + vol
			if isSamplePlaying( self.trailerSound.wheelSample ) then
				setSampleVolume( self.trailerSound.wheelSample, vol * TrailerSound.volScale )
			else
				playSample( self.trailerSound.wheelSample, 1, vol * TrailerSound.volScale, 0 )
			end
			setSamplePitch(  self.trailerSound.wheelSample, ptc * TrailerSound.ptcScale )
		elseif isSamplePlaying( self.trailerSound.wheelSample ) then
			stopSample( self.trailerSound.wheelSample )
		end
		if self.trailerSound.count > 0 and kmh >= 3 then
			if self.brakeLightsVisibility then
				if not self.trailerSound.brake then
					playSample( self.trailerSound.brakeSample, 1, 1, 0 )
					self.trailerSound.brake = true
				end
			elseif self.trailerSound.brake then
				self.trailerSound.brake = false
			end
	
			local vol = math.min( math.max( kmh / 25, 0 ), 1 )
			local ptc = vol * vol
			if isfield then
				vol = vol * 2
			end
			
			for i=2,self.trailerSound.count do
				vol = vol * 1.2
			end
			
			if isSamplePlaying( self.trailerSound.trailerSample ) then
				setSampleVolume( self.trailerSound.trailerSample, vol * TrailerSound.volScale )
			else
				playSample( self.trailerSound.trailerSample, 1, vol * TrailerSound.volScale, 0 )
			end
			setSamplePitch( self.trailerSound.trailerSample, ptc * TrailerSound.ptcScale )
		elseif isSamplePlaying( self.trailerSound.trailerSample ) then
			stopSample( self.trailerSound.trailerSample )
		end
	end
	
end;

function TrailerSound:draw()
end;

function TrailerSound:attachImplement(implement)
	--local object = implement.object
	--if SpecializationUtil.hasSpecialization(Trailer, object.specializations) then
	--	self.trailerSound.count = self.trailerSound.count + 1
	--end
end

function TrailerSound:detachImplement(implementIndex)
	--local object = self.attachedImplements[implementIndex].object
	--if SpecializationUtil.hasSpecialization(Trailer, object.specializations) then
	--	self.trailerSound.count = math.max( 0, self.trailerSound.count - 1 )
	--end
end

function TrailerSound.isField( x, z )
	for channel=0,3 do
		if Utils.getDensity(g_currentMission.terrainDetailId, channel, x-1, z-1, x+2, z, x, z+2 ) > 0 then
			return true
		end
	end
	
  setDensityReturnValueShift(id, -1)
  local ret = getDensityParallelogram(g_currentMission.fruits[FruitUtil.FRUITTYPE_GRASS].id, x-1, z-1, 2, 0, 0, 2, 0, g_currentMission.numFruitStateChannels)
  setDensityReturnValueShift(id, 0)
	if ret > 0 then
		return true
	end
	
	return false
end

function TrailerSound.countTrailers( vehicle )
	
	local count = 0
	if vehicle == nil then
		return 0
	end
	if SpecializationUtil.hasSpecialization(Trailer, vehicle.specializations) then
		count = 1
	end
	if vehicle.attachedImplements ~= nil and table.getn(vehicle.attachedImplements) > 0 then
		for _, v in pairs(vehicle.attachedImplements) do
			count = count + TrailerSound.countTrailers( v.object )
		end	
  end	
	
	return count
end
